Added 863 files from NVIDIA/cccl sparse checkout: - c2h/ (27 files): Catch2 test helpers — generators, validators, runner - nvbench_helper/ (10 files): Benchmark harness utilities - cmake/ (29 files): CMake presets and build helpers - cudax/ (794 files): Experimental CUDA extensions - AGENTS.md: NVIDIA's official AI agent instructions for CCCL - CMakePresets.json: Standardized build configurations - cccl-version.json: Version tracking Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to competition value and PRD items. cccl_upstream now covers 100% of competition-critical assets: - 27 tuning headers (SM80/90/100 benchmark data) - 32 dispatch headers (algorithm implementations) - 60 Thrust examples (correctness verification) - 217 CUB Catch2 tests (regression matrix) - 153 CUB benchmarks (parameter space search) - 18 CUB examples (API verification) - 27 test helpers + benchmark harness - 794 cudax experimental extensions
68 lines
1.9 KiB
Plaintext
68 lines
1.9 KiB
Plaintext
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of CUDASTF in CUDA C++ Core Libraries,
|
|
// under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
#include <cuda/experimental/__stf/stream/interfaces/slice_reduction_ops.cuh>
|
|
#include <cuda/experimental/__stf/stream/stream_ctx.cuh>
|
|
|
|
using namespace cuda::experimental::stf;
|
|
|
|
using scalar_t = slice_stream_interface<int, 1>;
|
|
|
|
template <typename T>
|
|
__global__ void set_value(T* addr, T val)
|
|
{
|
|
*addr = val;
|
|
}
|
|
|
|
template <typename T>
|
|
__global__ void check_value_and_reset(T* addr, T val, T reset_val)
|
|
{
|
|
assert(*addr == val);
|
|
*addr = reset_val;
|
|
}
|
|
|
|
template <typename T>
|
|
__global__ void add_val(T* inout_addr, T val)
|
|
{
|
|
*inout_addr += val;
|
|
}
|
|
|
|
int main()
|
|
{
|
|
stream_ctx ctx;
|
|
const int N = 4;
|
|
|
|
auto var_handle = ctx.logical_data(shape_of<slice<int>>(1));
|
|
var_handle.set_symbol("var");
|
|
|
|
auto redux_op = std::make_shared<slice_reduction_op_sum<int>>();
|
|
|
|
const int niters = 4;
|
|
for (int iter = 0; iter < niters; iter++)
|
|
{
|
|
// We add i (total = N(N-1)/2 + initial_value)
|
|
for (int i = 0; i < N; i++)
|
|
{
|
|
ctx.task(var_handle.relaxed(redux_op))->*[=](cudaStream_t stream, auto d_var) {
|
|
add_val<<<1, 1, 0, stream>>>(d_var.data_handle(), i);
|
|
cuda_safe_call(cudaGetLastError());
|
|
};
|
|
}
|
|
|
|
// Check that we have the expected value, and reset it so that we can perform another reduction
|
|
ctx.task(var_handle.rw())->*[=](cudaStream_t stream, auto d_var) {
|
|
int expected = (N * (N - 1)) / 2;
|
|
check_value_and_reset<<<1, 1, 0, stream>>>(d_var.data_handle(), expected, 0);
|
|
};
|
|
}
|
|
|
|
ctx.finalize();
|
|
}
|